//-----------------------------------------------------------------
// SX15-Evo Firmware di test con SX16B nuovo protocollo
// Data : 24-09-06
// Versione : 1.0
// Area SX
// http://www.areasx.com
// Note     : 
//-----------------------------------------------------------------

//-----------------------------------------------------------------
// Configurazione Firmware
//-----------------------------------------------------------------
//#define DEBUG_STRING
//#define DBG_STRING

#define MAX_SX16BOARDS    4

#define EINBUFSIZE   511
#define EOUTBUFSIZE  511

//-----------------------------------------------------------------
// Define Hardware
//-----------------------------------------------------------------
#define LEDLD1_ON       BitWrPortI(PFDR,&PFDRShadow,1,1)
#define LEDLD1_OFF      BitWrPortI(PFDR,&PFDRShadow,0,1)
#define LEDLD2_ON       BitWrPortI(PFDR,&PFDRShadow,1,0)
#define LEDLD2_OFF      BitWrPortI(PFDR,&PFDRShadow,0,0)
#define LEDLD3_ON       BitWrPortI(PFDR,&PFDRShadow,1,4)
#define LEDLD3_OFF      BitWrPortI(PFDR,&PFDRShadow,0,4)

#define KEY_PRESSED   BitRdPortI(PEDR,1)

// Output channel
#define STROBE        BitWrPortI(PBDR,&PBDRShadow,1,5);BitWrPortI(PBDR,&PBDRShadow,0,5)
#define CLOCK         BitWrPortI(PBDR,&PFDRShadow,1,4);BitWrPortI(PBDR,&PFDRShadow,0,4)
#define SET_DATA      BitWrPortI(PBDR,&PFDRShadow,1,7)
#define CLEAR_DATA    BitWrPortI(PBDR,&PFDRShadow,0,7)

// Input channel
#define INP_LOADHI    BitWrPortI(PBDR,&PBDRShadow,1,3)
#define INP_LOADLO    BitWrPortI(PBDR,&PBDRShadow,0,3)
#define INP_CLOCKHI   BitWrPortI(PBDR,&PBDRShadow,1,0)
#define INP_CLOCKLO   BitWrPortI(PBDR,&PBDRShadow,0,0)
#define INP_DATA      BitRdPortI(PBDR,2)

#define I2CSCLBit 5       //Linea di clock PD5
#define I2CSDABit 4       //Linea Data PD4

// Sposta il port dell'I2C sul PortD
#define i2c_SCL_H() BitWrPortI(PDDDR,&PDDDRShadow,0,I2CSCLBit)
#define i2c_SCL_L() BitWrPortI(PDDDR,&PDDDRShadow,1,I2CSCLBit)
#define i2c_SDA_H() BitWrPortI(PDDDR,&PDDDRShadow,0,I2CSDABit)
#define i2c_SDA_L() BitWrPortI(PDDDR,&PDDDRShadow,1,I2CSDABit)
#define i2c_SCL()   BitRdPortI(PDDR,I2CSCLBit)
#define i2c_SDA()   BitRdPortI(PDDR,I2CSDABit)

#define DS1621_ADDRESS 0x00      // Indirizzo del sensore di temperatura


//-----------------------------------------------------------------
// Liberie
//-----------------------------------------------------------------
#use "I2C.LIB"


//-----------------------------------------------------------------
// Globali
//-----------------------------------------------------------------
char rxBuff[512];
char txBuff[512];
char tmpbuf[500];
int nchar;
int i;
float Temperature;
float Humidity;
char CurrentOutput;
unsigned long CurrentInput;
char RFTxBuf[10];           // Buffer TX RF
char RFRxBuf[10];           // Buffer RX RF


//-----------------------------------------------------------------
// Prototipi
//-----------------------------------------------------------------
void BoardInit(void);
void msDelay(unsigned int);
void SetOutput (int, int );
void SetOutputNP (int, int, int );
void SetGlobalOutput (int );
cofunc float DS1621Read(unsigned char);
cofunc float DS1621ReadNP(unsigned char);
int DS1621Init(unsigned char);
cofunc unsigned long GetGlobalInput(void);
cofunc unsigned long GetGlobalInputNP(int);
int rSendRFCommand(unsigned long );
int GetSX16Number (void);
void SetSX16Number (int );
int GetOutStatusNP(int);


//--------------------------------------------------------
// Ritardo bloccante
// Riceve      : ams - numero di millisecondi da ritardare
// Restituisce : -
// Note        :
//--------------------------------------------------------
nodebug
void msDelay(unsigned int ams)
{
  auto unsigned long t0;
  for(t0=MS_TIMER; MS_TIMER<ams+t0;);
}

//--------------------------------------------------------
// Inizializza i port del modulo RCM
// Riceve      : -
// Restituisce : -
// Note        :
//--------------------------------------------------------
void BoardInit() {

  // Port A
  // Default ingresso

  // Port B
  // PB0 - ICL (output)
  BitWrPortI(PBDDR,&PBDDRShadow,1,0);
  // PB2 - IDA (input)
  // PB3 - IPL (output)
  BitWrPortI(PBDDR,&PBDDRShadow,1,3);
  // PB4 - OCL (output)
  BitWrPortI(PBDDR,&PBDDRShadow,1,4);
  // PB5 - STR (output)
  BitWrPortI(PBDDR,&PBDDRShadow,1,5);
  // PB7 - ODA (output)
  BitWrPortI(PBDDR,&PBDDRShadow,1,7);

  // Port C
  // PC0-PC1 - Seriale D
  BitWrPortI(PCFR, &PCFRShadow, 1,0);

  // PC2-PC3 - Seriale C
  // Modem GSM
  BitWrPortI(PCFR, &PCFRShadow, 1,2);

  // Port D
  // PD4 - I2C SDA
  BitWrPortI(PDDDR, &PDDDRShadow, 0, I2CSDABit);
  BitWrPortI(PDDR,&PDDRShadow,0,I2CSDABit);
  // PD5 - I2C SCL
  BitWrPortI(PDDDR, &PDDDRShadow, 0, I2CSCLBit);
  BitWrPortI(PDDR,&PDDRShadow,0,I2CSCLBit);

  // Port E
  WrPortI(PEFR, &PEFRShadow, 0x00);
  // PE0 - DCD Modem GSM (ingresso)
  // PE1 - Pulsante esterno (ingresso)
  // PE4 - GSM Power Control (ingresso)
  // PE5 - RS485 Data Direction / RS232 DTR (output)
  BitWrPortI(PEDDR, &PEDDRShadow, 1,5);
  // PE7 - GSM On/Off (output)
  BitWrPortI(PEDDR, &PEDDRShadow, 1,7);

  // Port F
  WrPortI(PFFR, &PFFRShadow, 0x00);
  WrPortI(PFDCR, &PFDCRShadow, 0x00);
  // PF0 - Led LD3 (output)
  BitWrPortI(PFDDR, &PFDDRShadow, 1,0);
  // PF1 - Led LD1 (output)
  BitWrPortI(PFDDR, &PFDDRShadow, 1,1);
  // PF4 - Led LD2 (output)
  BitWrPortI(PFDDR, &PFDDRShadow, 1,4);
  // PF5 - Modem Reset (output)
  BitWrPortI(PFDDR, &PFDDRShadow, 1,5);
  // PF6 - SH71 External I2c (bid)
  // PF7 - SH71 External I2c (bid)

  // Port G
  WrPortI(PGFR, &PGFRShadow, 0x00);
  // PG6-PG7 - Seriale E (RF Transmitter, Caller Id, GPS Receiver)
  BitWrPortI(PGFR, &PGFRShadow, 1,6);
  BitWrPortI(PGDDR, &PGDDRShadow, 0,7);
  BitWrPortI(PGDDR, &PGDDRShadow, 1,6);

}


//--------------------------------------------------------
// Trasmissione comandi verso il modulo RF
// Riceve      : comando in RFTxBuf, ser_timeout - Timeout seriale
// Restituisce : numero dei caratteri ricevuti
// Note        : La risposta  in RFRxBuf
//--------------------------------------------------------
int rSendRFCommand(unsigned long ser_timeout) {

  unsigned long ttimer;
  int i,nchar;

  serErdFlush();
  // Invio comando
  #ifdef DEBUG_STRING
    printf ("Invio comando\n--> ");
  #endif
  for (i=0;i<6;i++) {
    serEputc(RFTxBuf[i]);
    #ifdef DEBUG_STRING
      printf ("[%02X]",RFTxBuf[i]);
    #endif
  }
  #ifdef DEBUG_STRING
    printf ("\n");
  #endif

  ttimer=MS_TIMER;
  while ( (serEpeek()==-1) && (MS_TIMER-ttimer<ser_timeout) ) ;

  if (serEpeek()!=-1) {
    #ifdef DEBUG_STRING
      printf ("Ricezione dalla seriale RF\n");
    #endif
    nchar=serEread(RFRxBuf,6,500);
    #ifdef DEBUG_STRING
      printf ("Ricevuti [%d] caratteri RF\n<-- ",nchar);
      for (i=0;i<nchar;i++) {
        printf ("[%02X]",RFRxBuf[i]);
      }
      printf ("\n");
    #endif
    return nchar;

  } else {
    #ifdef DEBUG_STRING
      printf ("RF Serial TIMEOUT\n");
    #endif
    return 0;
  }

}


//--------------------------------------------------------
// Imposta le uscite
// Riceve      : outstatus - stato da impostare per le uscite
// Restituisce : -
// Note        :
//--------------------------------------------------------
void SetGlobalOutput (int outstatus) {

  auto int i;
  auto int dato;

    RFTxBuf[0]=0x33;
    RFTxBuf[1]=0x01;
    RFTxBuf[2]=0x03;
    RFTxBuf[3]=0xFF;
    RFTxBuf[4]=outstatus;
    RFTxBuf[5]=0;

    if (rSendRFCommand(500)) {
      printf ("Impostazione globali riuscite\n");
    } else {
      printf ("Impostazione globali timed out\n");
    }

}


//--------------------------------------------------------
// Imposta una uscita
// Riceve      : indice - indice dell'uscita da impostare (1-6)
//               val - stato da impostare (1 acceso, 0 spento)
// Restituisce : -
// Note        :
//--------------------------------------------------------
void SetOutput (int indice, int val) {

  if ((indice<1) || (indice>6)) {
    #ifdef DBG_STRING
      printf("Valore dell'indice errato\n");
    #endif
    return;
  }

  if (val)
    CurrentOutput |=(0x01<<(indice-1));
  else
    CurrentOutput &= ~(0x01<<(indice-1));

    RFTxBuf[0]=0x33;
    RFTxBuf[1]=0x01;
    RFTxBuf[2]=0x03;
    RFTxBuf[3]=indice;
    RFTxBuf[4]=val;
    RFTxBuf[5]=0x00;

    if (rSendRFCommand(500)) {
      printf ("Impostazione output riuscita\n");
    } else {
      printf ("Impostazione output timed out\n");
    }


}


// -----------------------------------------------------------------------
// Ricava lo stato delle uscite
// Riceve      : scheda - numero della scheda
// Restituisce : bitmap con lo stato delle uscite
// Note        :
// -----------------------------------------------------------------------
int GetOutStatusNP(int scheda) {
  
    RFTxBuf[0]=0x66;
    RFTxBuf[1]=0x01;
    RFTxBuf[2]=0x04;
    RFTxBuf[3]=scheda;
    RFTxBuf[4]=0x00;
    RFTxBuf[5]=0x00;

    if (rSendRFCommand(500)) {
      printf ("Lettura output (NP) riuscita\n");
      return RFRxBuf[4];
    } else {
      printf ("Lettura output (NP) timed out\n");
      return -1;
    }
  
}

//--------------------------------------------------------
// Imposta una uscita nuovo protocollo
// Riceve      : scheda - numero della scheda
//               indice - indice dell'uscita da impostare (1-6)
//               val - stato da impostare (1 acceso, 0 spento)
// Restituisce : -
// Note        :
//--------------------------------------------------------
void SetOutputNP (int scheda, int indice, int val) {

  if ( ((indice<1) || (indice>6)) && (indice!=0xFF) ) {
    #ifdef DBG_STRING
      printf("Valore dell'indice errato\n");
    #endif
    return;
  }

  if ( ((scheda<1) || (scheda>MAX_SX16BOARDS)) && (scheda!=0xFF) ) {
    #ifdef DBG_STRING
      printf("Valore della scheda errato\n");
    #endif
    return;
  }

  if (val)
    CurrentOutput |=(0x01<<(indice-1));
  else
    CurrentOutput &= ~(0x01<<(indice-1));

    RFTxBuf[0]=0x66;
    RFTxBuf[1]=0x01;
    RFTxBuf[2]=0x03;
    RFTxBuf[3]=scheda;
    RFTxBuf[4]=indice;
    RFTxBuf[5]=val;

    if (rSendRFCommand(500)) {
      printf ("Impostazione output (NP) riuscita\n");
    } else {
      printf ("Impostazione output (NP) timed out\n");
    }

}



//--------------------------------------------------------
// Imposta il numero di SX16 collegate
// Riceve      : numero - numero di SX16
// Restituisce : -
// Note        :
//--------------------------------------------------------
void SetSX16Number (int numero) {


    RFTxBuf[0]=0x66;
    RFTxBuf[1]=0x01;
    RFTxBuf[2]=0x08;
    RFTxBuf[3]=numero;
    RFTxBuf[4]=0;
    RFTxBuf[5]=0;

    if (rSendRFCommand(500)) {
      printf ("Impostazione numero SX16 riuscita\n");
    } else {
      printf ("Impostazione numero SX16 timed out\n");
    }

}


//--------------------------------------------------------
// Legge il numero di SX16 collegate
// Riceve      : numero - numero di SX16
// Restituisce : -
// Note        :
//--------------------------------------------------------
int GetSX16Number (void) {


    RFTxBuf[0]=0x66;
    RFTxBuf[1]=0x01;
    RFTxBuf[2]=0x09;
    RFTxBuf[3]=0;
    RFTxBuf[4]=0;
    RFTxBuf[5]=0;

    if (rSendRFCommand(500)) {
      printf ("Lettura del numero SX16 riuscita\n");
      return RFRxBuf[3];
    } else {
      printf ("Lettura del numero SX16 timed out\n");
    }

}



//--------------------------------------------------------
// Ricava gli ingressi
// Riceve      : -
// Restituisce : long con il bitmap degli ingressi
// Note        : In questa versione servono solo 16 ingressi,
//               quindi la funzione  customizzata
//--------------------------------------------------------
cofunc unsigned long GetGlobalInput(void) {

  auto int i;
  auto char latchval[4];
  unsigned long ret_val;

    RFTxBuf[0]=0x33;
    RFTxBuf[1]=0x01;
    RFTxBuf[2]=0x01;
    RFTxBuf[3]=0;
    RFTxBuf[4]=0;
    RFTxBuf[5]=0;
    i=rSendRFCommand(500);
    if (i==6) {
      latchval[0]=(RFRxBuf[3] ^ 0xFF);
      latchval[1]=(RFRxBuf[4] ^ 0xFF);
      latchval[2]=(RFRxBuf[5] ^ 0xFF);
      latchval[3]=0;
    } else {
      printf ("Lettura ingressi fallita\n");
    }


  #ifdef DEBUG_STRING
    printf ("Latches [%d][%d][%d][%d] - [%X][%X][%X][%X]\n",latchval[0],latchval[1],latchval[2],latchval[3],latchval[0],latchval[1],latchval[2],latchval[3]);
  #endif
  ret_val=((unsigned long)latchval[2]<<16) | ((unsigned long)latchval[1]<<8) | (latchval[0]);
  #ifdef DEBUG_STRING
    printf ("retval [%lu]\n",ret_val);
  #endif
  return ret_val;

}

//--------------------------------------------------------
// Ricava gli ingressi
// Riceve      : scheda - numero di scheda da leggere
// Restituisce : long con il bitmap degli ingressi
// Note        : In questa versione servono solo 16 ingressi,
//               quindi la funzione  customizzata
//--------------------------------------------------------
cofunc unsigned long GetGlobalInputNP(int scheda) {

  auto int i;
  auto char latchval[4];
  unsigned long ret_val;

  if ( ((scheda<1) || (scheda>MAX_SX16BOARDS)) ) {
    #ifdef DBG_STRING
      printf("Valore della scheda errato\n");
    #endif
    return -1;
  }

    RFTxBuf[0]=0x66;
    RFTxBuf[1]=0x01;
    RFTxBuf[2]=0x01;
    RFTxBuf[3]=scheda;
    RFTxBuf[4]=0;
    RFTxBuf[5]=0;
    i=rSendRFCommand(500);
    if (i==6) {
      latchval[0]=(RFRxBuf[3] ^ 0xFF);
      latchval[1]=(RFRxBuf[4] ^ 0xFF);
      latchval[2]=(RFRxBuf[5] ^ 0xFF);
      latchval[3]=0;
    } else {
      printf ("Lettura ingressi fallita\n");
    }

  #ifdef DEBUG_STRING
    printf ("Latches [%d][%d][%d][%d] - [%X][%X][%X][%X]\n",latchval[0],latchval[1],latchval[2],latchval[3],latchval[0],latchval[1],latchval[2],latchval[3]);
  #endif
  ret_val=((unsigned long)latchval[2]<<16) | ((unsigned long)latchval[1]<<8) | (latchval[0]);
  #ifdef DEBUG_STRING
    printf ("retval [%lu]\n",ret_val);
  #endif
  return ret_val;

}


// -----------------------------------------------------------------------
// Inizializzazione del sensore di temperatura
// Riceve     : address: indirizzo I2C del sensore
// Restituisce: 1 se tutto OK, <=0 se ci sono errori
// -----------------------------------------------------------------------
int DS1621Init(unsigned char address){

  auto short int err;

  i2c_stop_tx();

  // Invia lo start
  if (err=i2c_startw_tx()){
    i2c_stop_tx();
    return -1;
  }

  // Indirizza il chip e attende la risposta
  if (err=i2c_wr_wait(0x90 + (address<<1))){
    i2c_stop_tx();
    return -2;
  }

  // Parametri di configurazione del DS1621

  // Invia il comando ACh (Access Config)
  if (err=i2c_write_char(0xAC)){
    i2c_stop_tx();
    return -3;
  }

  // Invia il Dato 02h
  if (err=i2c_write_char(0x2)){
    i2c_stop_tx();
    return -4;
  }

  // Invia lo start di sincronismo
  if (err=i2c_startw_tx()) {
    i2c_stop_tx();
    return -5;
  }

  // Indirizza il chip e attende la risposta
  if (err=i2c_wr_wait(0x90 + (address<<1))) {
    i2c_stop_tx();
    return -6;
  }

  // Invia il comando EEh che avvia le conversioni
  if (err=i2c_write_char(0xEE)){
    i2c_stop_tx();
    return -7;
  }

  // Chiusura
  i2c_stop_tx();
  return 1;

}


// -----------------------------------------------------------------------
// Funzione di lettura temperatura dal DS1621
// Riceve     : address - Indirizzo I2C
// Restituisce: valore di temperatura letto (float) se tutto OK, un valore
//              <-100 se errore
// -----------------------------------------------------------------------
cofunc float DS1621Read(unsigned char address) {

  auto unsigned char cnt;
  auto short int err,i;
  auto float temp;

    RFTxBuf[0]=0x33;
    RFTxBuf[1]=0x01;
    RFTxBuf[2]=0x02;
    RFTxBuf[3]=0;
    RFTxBuf[4]=0;
    RFTxBuf[5]=0;

    i=rSendRFCommand(500);
    if (i) {
      // Converto in float
      temp=0;
      // Imposto il mezzo grado
      if (RFRxBuf[4]==0x80) temp=0.5;

      if (RFRxBuf[3] & 0x80) {
        // Temperatura negativa
        temp+=(!RFRxBuf[3])+1;
        temp=-temp;
      } else {
        // Temperatura >0
        temp+=RFRxBuf[3];
      }

      printf ("Temperatura letta %.1f\n",temp);

    } else {
      printf ("Lettura temperatura timed out\n");
    }


  return (temp);
}


// -----------------------------------------------------------------------
// Funzione di lettura temperatura dal DS1621
// Riceve     : address - Indirizzo I2C
// Restituisce: valore di temperatura letto (float) se tutto OK, un valore
//              <-100 se errore
// -----------------------------------------------------------------------
cofunc float DS1621ReadNP(unsigned char address) {

  auto unsigned char cnt;
  auto short int err,i;
  auto float temp;

    RFTxBuf[0]=0x66;
    RFTxBuf[1]=0x01;
    RFTxBuf[2]=0x02;
    RFTxBuf[3]=0;
    RFTxBuf[4]=0;
    RFTxBuf[5]=0;

    i=rSendRFCommand(500);
    if (i) {
      // Converto in float
      temp=0;
      // Imposto il mezzo grado
      if (RFRxBuf[4]==0x80) temp=0.5;

      if (RFRxBuf[3] & 0x80) {
        // Temperatura negativa
        temp+=(!RFRxBuf[3])+1;
        temp=-temp;
      } else {
        // Temperatura >0
        temp+=RFRxBuf[3];
      }

      printf ("Temperatura letta %.1f\n",temp);

    } else {
      printf ("Lettura temperatura timed out\n");
    }


  return (temp);
}


// -----------------------------------------------------------------------
// -----------------------------------------------------------------------
//                               MAIN
// -----------------------------------------------------------------------
// -----------------------------------------------------------------------
main () {

  char s[100];
  int cmd, scheda;

  // Inizializzazione Scheda
  BoardInit();

  // Impostazione allo stato di riposo di tutte le uscite
  SetGlobalOutput(0x00);

  // Inizializzazione libreria I2C
  i2c_init();

    // Apertura seriale RF
    serEopen(19200);

  printf ("SX15-Evo con SX16B: System Ready\n");


  while (1) {

    costate {
      // Lampeggio del led
      LEDLD1_ON;
      waitfor (DelayMs(500));
      LEDLD1_OFF;
      waitfor (DelayMs(500));
    }

    costate {

      printf ("\n\nCosa desideri fare?\n");
      printf ("------------------- Vecchio Protocollo ----------------------\n");
      printf ("1 - Lettura ingressi SX16\n");
      printf ("2 - Impostazione di una uscita SX16\n");
      printf ("3 - Lettura temperatura su SX16\n");

      printf ("------------------- Nuovo Protocollo ----------------------\n");
      printf ("4 - Lettura ingressi SX16\n");
      printf ("5 - Impostazione di una uscita SX16\n");
      printf ("6 - Lettura temperatura su SX16\n");
      printf ("7 - Impostazione del numero di SX16 collegate\n");
      printf ("8 - Lettura del numero di SX16 collegate\n");
      printf ("9 - Lettura dello stato delle uscite\n");

      printf("\nInserisci il comando da inviare e premi invio: ");
		  gets(s);
		  cmd = atoi(s);

      switch (cmd) {

      case 1:
        // Lettura ingressi
        printf ("Lettura degli ingressi su SX16\n");
        wfd CurrentInput=GetGlobalInput();

        printf ("---------------------------------------\n");
        printf ("Valore corrente degli ingressi:\n");
        for (i=0;i<24;i++) {
          if ( !(i % 8) || !(i % 16) ) {
            printf ("\n");
          }
          if (CurrentInput & (0x01L<<i)) {
            printf ("[%02d-ON ] ",i);
          } else {
            printf ("[%02d-OFF] ",i);
          }

        }
        printf ("\n---------------------------------------\n");

        break;

      case 2:
        // Impostazione uscite
        printf ("Numero dell'uscita da attivare (1-6)?\n");
	      gets(s);
	      cmd = atoi(s);

	      if ((cmd<1) || (cmd>6)) {
	        printf ("Errore. L'uscita deve essere compresa tra 1 e 6\n");
	      } else {
		      // Attivazione dell'uscita
		      printf ("---------------------------------------\n");
		      printf ("Attivo l'uscita per un secondo\n");
		      printf ("---------------------------------------\n");
		      SetOutput(cmd,1);
		      waitfor(DelayMs(1000));
		      SetOutput(cmd,0);
	      }


        break;

      case 3:
        // Lettura temperatura
        wfd Temperature=DS1621Read(0);

        printf ("---------------------------------------\n");
        printf ("Temperatura corrente: %.1f\n",Temperature);
        printf ("---------------------------------------\n");

        break;

      // -----------------------------------------------------------------------
      // Nuovo Protocollo
      // -----------------------------------------------------------------------
      case 4:
        // Lettura ingressi
        printf ("Lettura degli ingressi su SX16 (NP)\n");
        printf ("Numero della scheda (1-%d)?\n",MAX_SX16BOARDS);
		    gets(s);
		    scheda = atoi(s);
		    if ( ((scheda<1) || (scheda>MAX_SX16BOARDS)) ) {
		      printf ("Errore. La scheda deve essere compresa tra 1 e %d\n",MAX_SX16BOARDS);
		    }

        wfd CurrentInput=GetGlobalInputNP(scheda);

        printf ("---------------------------------------\n");
        printf ("Valore corrente degli ingressi (NP):\n");
        for (i=0;i<24;i++) {
          if ( !(i % 8) || !(i % 16) ) {
            printf ("\n");
          }
          if (CurrentInput & (0x01L<<i)) {
            printf ("[%02d-ON ] ",i);
          } else {
            printf ("[%02d-OFF] ",i);
          }

        }
        printf ("\n---------------------------------------\n");

        break;

      case 5:
        // Impostazione uscite
        printf ("Numero della scheda (1-%d o 0xFF per broadcast)?\n",MAX_SX16BOARDS);
		    gets(s);
		    scheda = atoi(s);
		    if ( ((scheda<1) || (scheda>MAX_SX16BOARDS)) && (scheda!=0xFF) ) {
		      printf ("Errore. La scheda deve essere compresa tra 1 e %d\n",MAX_SX16BOARDS);
		    }

        if (scheda!=0xFF) {
          printf ("Numero dell'uscita da attivare (1-6)?\n");
		      gets(s);
		      cmd = atoi(s);
		    } else {
		      cmd=1;
		    }
	      if ( ((cmd<1) || (cmd>6)) && (cmd!=0xFF) ) {
	        printf ("Errore. L'uscita deve essere compresa tra 1 e 6\n");
	      } else {
		      // Attivazione dell'uscita
		      printf ("---------------------------------------\n");
		      printf ("Attivo l'uscita per un secondo (NP)\n");
		      printf ("---------------------------------------\n");
		      SetOutputNP(scheda,cmd,1);
		      waitfor(DelayMs(1000));
		      SetOutputNP(scheda,cmd,0);
	      }

        break;

      case 6:
        // Lettura temperatura
        wfd Temperature=DS1621ReadNP(0);

        printf ("---------------------------------------\n");
        printf ("Temperatura corrente (NP): %.1f\n",Temperature);
        printf ("---------------------------------------\n");

        break;

      case 7:
        // Impostazione numero SX 16
        printf ("Numero di SX16 collegate (1-%d)?\n",MAX_SX16BOARDS);
		    gets(s);
		    cmd = atoi(s);

	      // Impostazione
	      SetSX16Number(cmd);

        break;

      case 8:
        // Lettura numero SX 16		      
		    cmd=GetSX16Number();
		    printf ("Numero di SX16 configurate: [%d]\n",cmd);		    

        break;

      case 9:
        // Lettura stato delle uscite
        printf ("Numero della scheda (1-%d)?\n",MAX_SX16BOARDS);
		    gets(s);
		    scheda = atoi(s);
		    if ( ((scheda<1) || (scheda>MAX_SX16BOARDS)) ) {
		      printf ("Errore. La scheda deve essere compresa tra 1 e %d\n",MAX_SX16BOARDS);
		    }        
		    cmd=GetOutStatusNP(scheda);
		    printf ("Stato delle uscite: [%02X]\n",cmd);

        break;


      default:
        printf ("Comando sconosciuto\n\n");
        break;

      }


    }

  }

}